package com.iohao.mmo.equip.action;

import com.iohao.game.action.skeleton.annotation.ActionController;
import com.iohao.game.action.skeleton.annotation.ActionMethod;
import com.iohao.game.action.skeleton.core.flow.FlowContext;
import com.iohao.game.action.skeleton.protocol.wrapper.StringValue;
import com.iohao.mmo.common.config.GameCode;
import com.iohao.mmo.equip.cmd.EquipCmd;
import com.iohao.mmo.equip.entity.Equip;
import com.iohao.mmo.equip.mapper.EquipMapper;
import com.iohao.mmo.equip.proto.*;
import com.iohao.mmo.equip.service.EquipService;
import com.iohao.mmo.equip.service.EquipTemplateService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import java.math.BigDecimal;
import java.util.List;

/**
 * @author 唐斌
 * @date 2023-07-30
 * @description: 装备类
 */
@Slf4j
@Component
@ActionController(EquipCmd.cmd)
public class EquipAction {
    @Resource
    EquipService equipService;
    @Resource
    EquipTemplateService equipTemplateService;

    /**
     * 获取某人装备列表信息
     *
     * @param flowContext flowContext
     */
    @ActionMethod(EquipCmd.getEquipList)
    public List<EquipMessage> getEquipList(FlowContext flowContext) {
        long userId = flowContext.getUserId();
        List<Equip> equipList = equipService.listByUser(userId);
        return EquipMapper.ME.convert(equipList);
    }

    /**
     * 获取装备信息
     *
     * @param stringValue itemTypeId
     */
    @ActionMethod(EquipCmd.getEquip)
    public EquipMessage getEquip(StringValue stringValue) {
        Equip equip = equipService.findById(stringValue.value);
        return EquipMapper.ME.convert(equip);
    }

    /**
     * 重新随机装备基础属性
     *
     * @param stringValue 重新随机装备基础属性参数对象
     */
    @ActionMethod(EquipCmd.resetEquipBasic)
    public EquipShowMessage resetEquipBasic(StringValue stringValue) {
        GameCode.objNotFound.assertTrue(
                StringUtils.isNotBlank(stringValue.value)
        );
        Equip equip = equipService.resetEquipBasic(stringValue.value);
        return EquipMapper.ME.convertShowMessage(equip);
    }

    /**
     * 重新随机词条
     *
     * @param equipResetMessage 重新随机装备总属性点参数对象
     */
    @ActionMethod(EquipCmd.resetEquip)
    public EquipShowMessage resetEquip(EquipResetMessage equipResetMessage) {
        GameCode.objNotFound.assertTrue(
                StringUtils.isNotBlank(equipResetMessage.id)
        );
        Equip equip = equipService.resetEquip(
                equipResetMessage.id,
                StringUtils.isNotBlank(equipResetMessage.excellentRateString)?
                        new BigDecimal(equipResetMessage.excellentRateString):BigDecimal.ZERO
        );
        return EquipMapper.ME.convertShowMessage(equip);
    }

    /**
     * 批量删除装备
     *
     * @param idList 装备id列表
     */
    @ActionMethod(EquipCmd.delEquipBatch)
    public void delEquipBatch(List<String> idList) {
        equipService.delBatch(idList);
    }

    /**
     * 通过材料创建新的装备-内部调用
     * @param createEquipMessage 创建新装备的入参
     * @param flowContext flowContext
     * @return
     */
    @ActionMethod(EquipCmd.createEquip)
    public NewEquipMessage internalCreateEquip(CreateEquipMessage createEquipMessage, FlowContext flowContext) {
        // 玩家
        long userId = flowContext.getUserId();

        Equip equip = equipTemplateService.randomEquip(
                createEquipMessage.itemTypeId,
                userId);

        /*
         * 将新装备信息给到调用方，
         * 装备模块并不关心调用方是谁，只要调用此 action，就生成新装备。
         */

        NewEquipMessage newEquipMessage = new NewEquipMessage();
        newEquipMessage.equipId = equip.getId();
        newEquipMessage.itemTypeId = equip.getItemTypeId();

        return newEquipMessage;
    }
}
