/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.service;

import com.iohao.mmo.common.config.GameCode;
import com.iohao.mmo.common.kit.RandomKit;
import com.iohao.mmo.equip.entity.*;
import lombok.AllArgsConstructor;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 装备库
 *
 * @author 唐斌
 * @date 2023-07-30
 * @description: 装备库实现类
 */
@Service
@AllArgsConstructor
public class EquipTemplateService {

    final MongoTemplate mongoTemplate;

    final EquipQualityService equipQualityService;

    final EquipEntryTemplateService equipEntryTemplateService;

    /**
     * 通过id查询装备库
     * @param id 装备库id
     * @return
     */
    public EquipTemplate findById(String id) {
        return  mongoTemplate.findById(id, EquipTemplate.class);
    }

    /**
     * 通过itemTypeId查询装备库
     * @param itemTypeId 物品分类标识
     * @return
     */
    public EquipTemplate findByItemTypeId(String itemTypeId) {
        return  mongoTemplate.findOne(
                Query.query(new Criteria("itemTypeId").is(itemTypeId)), EquipTemplate.class);
    }

    /**
     * 保存装备库
     * @param equipTemplate 装备库
     */
    public void save(EquipTemplate equipTemplate) {
        mongoTemplate.save(equipTemplate);
    }

    /**
     * 根据装备库id列表批量删除装备库
     * @param idList 装备库id列表
     */
    public void delBatch(List<String> idList) {
        Query query = new Query(new Criteria("id").in(idList));
        mongoTemplate.remove(query, EquipTemplate.class);
    }

    /**
     * 根据装备库列表批量随机新的装备
     * @param itemTypeIdList 物品分类标识列表
     * @param userId 用户id
     * @return
     */
    public List<Equip> randomEquipBatch(List<String> itemTypeIdList,long userId){
        List<Equip> equipList = itemTypeIdList.stream()
                .map(itemTypeId -> randomEquip(itemTypeId,userId))
                .toList();
        return equipList;
    }

    /**
     * 根据装备库随机一件新的装备
     * @param itemTypeId itemTypeId
     * @param userId 用户id
     * @return
     */
    public Equip randomEquip(String itemTypeId, long userId){
        //取到装备库
        EquipTemplate equipTemplate = findByItemTypeId(itemTypeId);
        GameCode.objNotFound.assertTrue(equipTemplate!=null,"找不到对应装备库");
        //将装备库属性拷贝到装备中
        Equip equip = copyTemplate2Equip(equipTemplate);
        equip.setUserId(userId);
        mongoTemplate.save(equip);
        return equip;
    }

    /**
     * 根据需要拷贝装备库属性到装备
     * @param equipTemplate 装备库
     * @return
     */
    public Equip copyTemplate2Equip(EquipTemplate equipTemplate){

        List<EquipEntry> equipPropertyFixedList = equipTemplate.getEquipPropertyFixedList();
        int fixedCount = 0;
        //如果没有预设固定属性，那么就是随机固定属性的装备
        if(equipPropertyFixedList==null|| equipPropertyFixedList.isEmpty()){
            fixedCount = RandomKit.randomFromInt(
                    equipTemplate.getEquipPropertyFixedMin(),
                    equipTemplate.getEquipPropertyFixedMax()+1);
            if(fixedCount>0) {
                //生成随机固定属性
                equipPropertyFixedList =
                        equipEntryTemplateService.randomEntryBatch(
                                equipTemplate.getLevel(),
                                equipTemplate.getPosition(),
                                fixedCount);
            }
        }
        //随机固定属性的具体值
        resetFixedEntryValue(equipTemplate.getLevel(), equipPropertyFixedList);

        //生成随机属性
        List<EquipEntry> equipEntryList =
                resetRandomEntryList(
                        equipTemplate.getQuality(),
                        equipTemplate.getLevel(),
                        equipTemplate.getPosition()
                );

        return Equip.builder()
                .itemTypeId(equipTemplate.getItemTypeId())
                .position(equipTemplate.getPosition())
                .level(equipTemplate.getLevel())
                .quality(equipTemplate.getQuality())
                .durability(equipTemplate.getDurability())
                .nowDurability(equipTemplate.getDurability())
                .identifyBasicCount(0)
                .identifyEntryCount(0)
                .equipPropertyBasicMin(equipTemplate.getEquipPropertyBasicMin())
                .equipPropertyBasicMax(equipTemplate.getEquipPropertyBasicMax())
                .equipPropertyBasic(
                        EquipPropertyBasic.randomBasic(
                            equipTemplate.getEquipPropertyBasicMin(),
                            equipTemplate.getEquipPropertyBasicMax()))
                .equipPropertyFixedList(equipPropertyFixedList)
                .equipPropertyRandomList(equipEntryList)
                .build();
    }

    /**
     * 重新随机固定属性的具体值
     * @param level 装备要求等级
     * @param equipPropertyFixedList 固定属性词条
     * @return List<EquipEntry> 装备词条列表
     */
    public void resetFixedEntryValue(int level,List<EquipEntry> equipPropertyFixedList){
        if(equipPropertyFixedList!=null&&!equipPropertyFixedList.isEmpty()) {
            //生成准确的固定属性
            equipEntryTemplateService.propertyDetail(
                    level,
                    equipPropertyFixedList);
        }
    }

    /**
     * 重新生成随机属性
     * @param quality 装备品质
     * @param level 装备要求等级
     * @param position 部位
     * @return List<EquipEntry> 装备词条列表
     */
    public List<EquipEntry> resetRandomEntryList(String quality,int level,String position){
        List<EquipEntry> equipEntryList = new ArrayList<>();

        EquipQuality equipQuality = equipQualityService.findByQuality(quality);
        if (equipQuality==null||(equipQuality.getRandomNumMax()==0)){
            return equipEntryList;
        }
        //随机属性的条数
//        int randomCount = RandomKit.randomFromInt(
//                equipQuality.getRandomNumMin(),
//                equipQuality.getRandomNumMax()+1);

        int randomCount = equipQuality.getRandomNumMin() + RandomKit.weightRandom(equipQuality.getPrizeWeightArr());
        //生成随机属性
        if(randomCount>0) {
            equipEntryList = equipEntryTemplateService.randomEntryBatch(
                    level,
                    position,
                    randomCount);
        }
        if(equipEntryList!=null&&!equipEntryList.isEmpty()) {
            //生成准确的随机属性
            equipEntryTemplateService.propertyDetail(
                    level,
                    equipEntryList);
        }
        return equipEntryList;
    }
}
